(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: generator_spec.mli,v 1.9 2014/01/31 09:51:52 fclement Exp $ *)

(** Generates the abstract syntax tree of a function that
   walks a type definition (a.k.a. a type definition walker or simply
   a walker).

   Generates names for the walkers and for their arguments; in case of
   polymorphic type definitions, also generates the function names of the
   walker arguments and their applications to recursively walk the type
   arguments.
*)

module type Prefix =

sig

  val function_prefix : string;;
  val type_var_arg_prefix : string;;
  val external_module_suffix : Og_file.file_suffix;;
  val output_file_suffix : Og_file.file_suffix;;
  val buffer_argument : (string * Og_types.type_expr) option;;
  val projection_prefix : string;;
  val injection_prefix : string;;

end
;;
(** For instance, to generate printers we could use:
   module Prefix = struct
     let function_prefix = "print_";;
     let type_var_arg_prefix = "pr_";;
     let external_module_suffix = "_print";;
     let buffer_argument =
       Some ("ppf", Const_type (Ldot (Lident "Format", "formatter")));;
   end
   ;;
*)

(** The signature of a walker generator. *)
module type Walker_ast_generator =

sig

  val file : Og_types.file -> Walker_ast.file;;

end
;;

(** The signature of a walker generator maker. *)
module type Make_ast_walker_generator =
 functor (P : Prefix) -> Walker_ast_generator
;;

module type Make =
 functor (P : Prefix) -> Walker_ast_generator
;;

(*
module P =
struct
  let function_prefix = "print_";;
  let type_var_arg_prefix = "pr_";;
  let external_module_suffix = "_print";;
  let buffer_argument = Some "ppf";;
end
;;

let walker_ast_generator =
  Generator_spec.Make (P : Prefix).file : Og_types.file -> Walker_ast.file
;;

let walker_ast =
  let src_ast =
    let ic = ...
    let parse_channel = ...
    parse_channel ic in
  walker_ast_generator src_ast

 Walker_ast_print.print_file
;;
*)
