(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: caml_print_xml.ml,v 1.10 2014/01/31 09:51:53 fclement Exp $ *)

module P : Generator_spec.Prefix =

struct

  let function_prefix = "print_xml_";;
  let type_var_arg_prefix = "px_";;
  let external_module_suffix = "_print_xml";;
  let output_file_suffix = "_print_xml";;
  let buffer_argument =
    Some ("ppf",
      Og_types.Const_type (
        Og_types.Ldot (Og_types.Lident "Format", "formatter")
      )
    )
  ;;
  let injection_prefix = "of_";;
  let projection_prefix = "to_";;

end
;;

(* The module that builds the walker ast. *)
module Make_walker = Generator.Make (P);;

module Treat (Io : Walk_spec.Out) =

struct

  (* The module that generates printers *)
  module Walker = Walk_print_xml.Printer (Io);;

  let file ast =
    (* Prints the type definition, if necessary. *)
    if Og_config.get_keep_type_definitions ()
      then Og_types_print.file Io.buffer ast;
    (* And the printer definition. *)
    let walker_ast = Make_walker.file ast in
    Walker.file walker_ast
  ;;

end
;;

module Main = Walk_file.Main (P) (Treat);;

Printexc.catch Main.main ();;
